@@ -88,6 +88,8 @@ char *stpcpy ();
 /* #define NDEBUG 1 */
 #include <assert.h>
 
+#include "save-cwd.h"
+
 #ifndef _LIBC
 # undef __chdir
 # define __chdir chdir
@@ -95,9 +97,6 @@ char *stpcpy ();
 # define __closedir closedir
 # undef __fchdir
 # define __fchdir fchdir
-# undef __getcwd
-# define __getcwd(P, N) xgetcwd ()
-extern char *xgetcwd (void);
 # undef __mempcpy
 # define __mempcpy mempcpy
 # undef __opendir
@@ -138,7 +137,7 @@ int rpl_lstat (const char *, struct stat *);
 # define FTW_NAME ftw
 # define NFTW_NAME nftw
 # define INO_T ino_t
-# define STAT stat
+# define FTW_STAT stat
 # ifdef _LIBC
 #  define LXSTAT __lxstat
 #  define XSTAT __xstat
@@ -158,6 +157,22 @@ int rpl_lstat (const char *, struct stat *);
 # define PATH_MAX 1024
 #endif
 
+#ifndef S_IFMT
+# define S_IFMT 0170000
+#endif
+
+#if STAT_MACROS_BROKEN
+# undef S_ISLNK
+#endif
+
+#ifndef S_ISLNK
+# ifdef S_IFLNK
+#  define S_ISLNK(m) (((m) & S_IFMT) == S_IFLNK)
+# else
+#  define S_ISLNK(m) 0
+# endif
+#endif
+
 struct dir_data
 {
   DIR *stream;
@@ -209,17 +224,31 @@ struct ftw_data
    as `ftw', map each flag to the subset of values used by `ftw'.  */
 static const int nftw_arr[] =
 {
-  FTW_F, FTW_D, FTW_DNR, FTW_NS, FTW_SL, FTW_DP, FTW_SLN
+  /* Extract with:
+     perl -ne '/^  (FTW_\w+)\b[^=]*$/ and print "$1, "' ftw_.h; echo */
+  FTW_F, FTW_D, FTW_DNR, FTW_NS, FTW_DCH, FTW_DCHP, FTW_DPRE, FTW_SL, FTW_DP, FTW_SLN
 };
 
 static const int ftw_arr[] =
 {
-  FTW_F, FTW_D, FTW_DNR, FTW_NS, FTW_F, FTW_D, FTW_NS
+  /* Generate with:
+     perl -ne '/^  (FTW_\w+)\b[^=]*$/ and print "$1, "' ftw_.h \
+       | sed 's/_SL/_F/;s/_DP\>/_D/;s/_SLN/_NS/'; echo  */
+  FTW_F, FTW_D, FTW_DNR, FTW_NS, FTW_DCH, FTW_DCHP, FTW_DPRE, FTW_F, FTW_D, FTW_NS
+};
+
+/* Ensure that each of the above arrays has the number of
+   initializers corresponding to enum Ftw_option.  */
+struct dummy
+{
+  int p1 [sizeof nftw_arr / sizeof nftw_arr[0] == FTW_N_MEMBERS ? 1 : -1];
+  int p2 [sizeof nftw_arr / sizeof  ftw_arr[0] == FTW_N_MEMBERS ? 1 : -1];
 };
 
 
 /* Forward declarations of local functions.  */
-static int ftw_dir (struct ftw_data *data, struct STAT *st) internal_function;
+static int ftw_dir (struct ftw_data *data, struct FTW_STAT *st)
+     internal_function;
 
 
 static int
@@ -238,7 +267,7 @@ object_compare (const void *p1, const void *p2)
 
 
 static inline int
-add_object (struct ftw_data *data, struct STAT *st)
+add_object (struct ftw_data *data, struct FTW_STAT *st)
 {
   struct known_object *newp = malloc (sizeof (struct known_object));
   if (newp == NULL)
@@ -250,7 +279,7 @@ add_object (struct ftw_data *data, struct STAT *st)
 
 
 static inline int
-find_object (struct ftw_data *data, struct STAT *st)
+find_object (struct ftw_data *data, struct FTW_STAT *st)
 {
   struct known_object obj;
   obj.dev = st->st_dev;
@@ -354,7 +383,7 @@ static inline int
 process_entry (struct ftw_data *data, struct dir_data *dir, const char *name,
 	       size_t namlen)
 {
-  struct STAT st;
+  struct FTW_STAT st;
   int result = 0;
   int flag = 0;
   size_t new_buflen;
@@ -466,7 +495,7 @@ process_entry (struct ftw_data *data, struct dir_data *dir, const char *name,
 
 static int
 internal_function
-ftw_dir (struct ftw_data *data, struct STAT *st)
+ftw_dir (struct ftw_data *data, struct FTW_STAT *st)
 {
   struct dir_data dir;
   struct dirent64 *d;
@@ -576,8 +605,9 @@ ftw_dir (struct ftw_data *data, struct STAT *st)
       __set_errno (save_err);
     }
 
-  /* Prepare the return, revert the `struct FTW' information.  */
-  data->dirbuf[data->ftw.base - 1] = '\0';
+  /* Prepare the return, revert the `struct FTW' information.
+     If ftw.base is 1, be careful to preserve the `/'.  */
+  data->dirbuf[data->ftw.base == 1 ? 1 : data->ftw.base - 1] = '\0';
   --data->ftw.level;
   data->ftw.base = previous_base;
 
@@ -636,14 +666,14 @@ base_name (char const *name)
 
 static int
 internal_function
-ftw_startup (const char *dir, int is_nftw, void *func, int descriptors,
+ftw_startup (const char *dir, int is_nftw, NFTW_FUNC_T func, int descriptors,
 	     int flags)
 {
   struct ftw_data data;
-  struct STAT st;
+  struct FTW_STAT st;
   int result = 0;
   int save_err;
-  char *cwd = NULL;
+  struct saved_cwd cwd;
   size_t dir_len;
 
   /* First make sure the parameters are reasonable.  */
@@ -682,7 +712,7 @@ ftw_startup (const char *dir, int is_nftw, void *func, int descriptors,
      every case the callback using the format of the `nftw' version
      and get the correct result since the stack layout for a function
      call in C allows this.  */
-  data.func = (NFTW_FUNC_T) func;
+  data.func = func;
 
   /* Since we internally use the complete set of FTW_* values we need
      to reduce the value range before calling a `ftw' callback.  */
@@ -694,9 +724,7 @@ ftw_startup (const char *dir, int is_nftw, void *func, int descriptors,
   /* Now go to the directory containing the initial file/directory.  */
   if (flags & FTW_CHDIR)
     {
-      /* GNU extension ahead.  */
-      cwd =  __getcwd (NULL, 0);
-      if (cwd == NULL)
+      if (save_cwd (&cwd))
 	result = -1;
       else if (data.ftw.base > 0)
 	{
@@ -772,11 +800,22 @@ ftw_startup (const char *dir, int is_nftw, void *func, int descriptors,
     }
 
   /* Return to the start directory (if necessary).  */
-  if (cwd != NULL)
+  if (flags & FTW_CHDIR)
     {
       save_err = errno;
-      __chdir (cwd);
-      free (cwd);
+      /* If restore_cwd fails and there wasn't a prior failure,
+	 then let this new errno override any prior value.
+	 FIXME: ideally, we'd be able to return some indication
+	 of what the failure means.  Otherwise, the caller will
+	 have a hard time distinguishing between e.g., `out of memory'
+	 and this sort of failure.  */
+      if (restore_cwd (&cwd) && result == 0)
+	{
+	  save_err = errno;
+	  result = -1;
+	}
+      free_cwd (&cwd);
+
       __set_errno (save_err);
     }
 
@@ -799,7 +838,7 @@ FTW_NAME (path, func, descriptors)
      FTW_FUNC_T func;
      int descriptors;
 {
-  return ftw_startup (path, 0, func, descriptors, 0);
+  return ftw_startup (path, 0, (NFTW_FUNC_T) func, descriptors, 0);
 }
 
 int
