@@ -40,6 +40,7 @@
 #include "path-concat.h"
 #include "quote.h"
 #include "same.h"
+#include "utimens.h"
 #include "xreadlink.h"
 
 #define DO_CHOWN(Chown, File, New_uid, New_gid)				\
@@ -425,8 +426,8 @@ close_src_desc:
    work to do and should return successfully, right away.
 
    Set *UNLINK_SRC if we've determined that the caller wants to do
-   `rename (a, b)' where `a' and `b' are hard links to the same file.
-   In that case, the caller should try to unlink `a' and then return
+   `rename (a, b)' where `a' and `b' are distinct hard links to the same
+   file. In that case, the caller should try to unlink `a' and then return
    successfully.  Ideally, we wouldn't have to do that, and we'd be
    able to rely on rename to remove the source file.  However, POSIX
    mistakenly requires that such a rename call do *nothing* and return
@@ -560,7 +561,9 @@ same_file_ok (const char *src_path, const struct stat *src_sb,
       if (S_ISLNK (dst_sb_link->st_mode))
 	return 1;
 
-      if (same_link && !same_name (src_path, dst_path))
+      if (same_link
+	  && 1 < dst_sb_link->st_nlink
+	  && ! same_name (src_path, dst_path))
 	{
 	  if (x->move_mode)
 	    {
@@ -1523,16 +1526,14 @@ copy_internal (const char *src_path, const char *dst_path,
 
   if (x->preserve_timestamps)
     {
-      struct utimbuf utb;
+      struct timespec timespec[2];
 
-      /* There's currently no interface to set file timestamps with
-	 better than 1-second resolution, so discard any fractional
-	 part of the source timestamp.  */
+      timespec[0].tv_sec = src_sb.st_atime;
+      timespec[0].tv_nsec = TIMESPEC_NS (src_sb.st_atim);
+      timespec[1].tv_sec = src_sb.st_mtime;
+      timespec[1].tv_nsec = TIMESPEC_NS (src_sb.st_mtim);
 
-      utb.actime = src_sb.st_atime;
-      utb.modtime = src_sb.st_mtime;
-
-      if (utime (dst_path, &utb))
+      if (utimens (dst_path, timespec))
 	{
 	  error (0, errno, _("preserving times for %s"), quote (dst_path));
 	  if (x->require_preserve)
