@@ -1,5 +1,5 @@
 /* dd -- convert a file while copying it.
-   Copyright (C) 85, 90, 91, 1995-2002 Free Software Foundation, Inc.
+   Copyright (C) 85, 90, 91, 1995-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <getopt.h>
 
 #include "system.h"
-#include "closeout.h"
 #include "error.h"
 #include "full-write.h"
 #include "getpagesize.h"
@@ -307,8 +306,8 @@ Copy a file, converting and formatting according to the options.\n\
       fputs (_("\
 \n\
 BLOCKS and BYTES may be followed by the following multiplicative suffixes:\n\
-xM M, c 1, w 2, b 512, kB 1000, K 1024, MB 1,000,000, M 1,048,576,\n\
-GB 1,000,000,000, G 1,073,741,824, and so on for T, P, E, Z, Y.\n\
+xM M, c 1, w 2, b 512, kB 1000, K 1024, MB 1000*1000, M 1024*1024,\n\
+GB 1000*1000*1000, G 1024*1024*1024, and so on for T, P, E, Z, Y.\n\
 Each KEYWORD may be:\n\
 \n\
 "), stdout);
@@ -327,6 +326,17 @@ Each KEYWORD may be:\n\
   noerror   continue after read errors\n\
   sync      pad every input block with NULs to ibs-size; when used\n\
               with block or unblock, pad with spaces rather than NULs\n\
+"), stdout);
+      fputs (_("\
+\n\
+Note that sending a SIGUSR1 signal to a running `dd' process makes it\n\
+print to standard error the number of records read and written so far,\n\
+then to resume copying.\n\
+\n\
+  $ dd if=/dev/zero of=/dev/null& pid=$!\n\
+  $ kill -USR1 $pid; sleep 1; kill $pid\n\
+  10899206+0 records in\n\
+  10899206+0 records out\n\
 "), stdout);
       printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
@@ -1141,6 +1151,7 @@ main (int argc, char **argv)
   int i;
   int exit_status;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
