@@ -46,7 +46,7 @@
 #define PROGRAM_NAME "du"
 
 #define AUTHORS \
-  N_ ("Torbjorn Granlund, David MacKenzie, Larry McVoy, Paul Eggert, and Jim Meyering")
+  N_ ("Torbjorn Granlund, David MacKenzie, Paul Eggert, and Jim Meyering")
 
 /* Initial size of the hash table.  */
 #define INITIAL_TABLE_SIZE 103
@@ -214,7 +214,7 @@ Mandatory arguments to long options are mandatory for short options too.\n\
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
       fputs (_("\n\
 SIZE may be (or may be an integer optionally followed by) one of following:\n\
-kB 1000, K 1024, MB 1,000,000, M 1,048,576, and so on for G, T, P, E, Z, Y.\n\
+kB 1000, K 1024, MB 1000*1000, M 1024*1024, and so on for G, T, P, E, Z, Y.\n\
 "), stdout);
       printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
@@ -251,7 +251,7 @@ hash_ins (ino_t ino, dev_t dev)
   struct entry *ent;
   struct entry *ent_from_table;
 
-  ent = (struct entry *) xmalloc (sizeof *ent);
+  ent = xmalloc (sizeof *ent);
   ent->st_ino = ino;
   ent->st_dev = dev;
 
@@ -574,6 +574,7 @@ main (int argc, char **argv)
   cwd_only[0] = ".";
   cwd_only[1] = NULL;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -642,7 +643,7 @@ main (int argc, char **argv)
 		     quote (optarg));
 	      fail = 1;
 	    }
- 	  break;
+	  break;
 
 	case 'm': /* obsolescent */
 	  human_output_opts = 0;
@@ -725,7 +726,7 @@ main (int argc, char **argv)
   if (opt_summarize_only)
     max_depth = 0;
 
-  files = (optind == argc ? cwd_only : argv + optind);
+  files = (optind < argc ? argv + optind : cwd_only);
 
   /* Initialize the hash structure for inode numbers.  */
   hash_init ();
