@@ -28,7 +28,6 @@
 
 #include "system.h"
 #include "error.h"
-#include "closeout.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "id"
@@ -120,6 +119,7 @@ main (int argc, char **argv)
   /* If nonzero, output only the user ID(s). -u */
   int just_user = 0;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -262,7 +262,7 @@ xgetgroups (const char *username, gid_t gid, int *n_groups,
     max_n_groups = getugroups (0, NULL, username, gid);
 
   /* Add 1 just in case max_n_groups is zero.  */
-  g = (GETGROUPS_T *) xmalloc (max_n_groups * sizeof (GETGROUPS_T) + 1);
+  g = xmalloc (max_n_groups * sizeof (GETGROUPS_T) + 1);
   if (username == 0)
     ng = getgroups (max_n_groups, g);
   else
