@@ -25,7 +25,6 @@
 #include <getopt.h>
 
 #include "system.h"
-#include "closeout.h"
 #include "error.h"
 #include "hard-locale.h"
 #include "linebuffer.h"
@@ -189,9 +188,9 @@ ADD_FIELD (struct line *line, const unsigned char *field, size_t len)
   if (line->nfields >= line->nfields_allocated)
     {
       line->nfields_allocated = (3 * line->nfields_allocated) / 2 + 1;
-      line->fields = (struct field *) xrealloc ((char *) line->fields,
-						(line->nfields_allocated
-						 * sizeof (struct field)));
+      line->fields = xrealloc (line->fields,
+			       (line->nfields_allocated
+				* sizeof (struct field)));
     }
   line->fields[line->nfields].beg = field;
   line->fields[line->nfields].len = len;
@@ -260,7 +259,7 @@ get_line (FILE *fp, struct line *line)
 {
   initbuffer (&line->buf);
 
-  if (! readline (&line->buf, fp))
+  if (! readlinebuffer (&line->buf, fp))
     {
       free (line->buf.buffer);
       line->buf.buffer = NULL;
@@ -277,7 +276,7 @@ get_line (FILE *fp, struct line *line)
 static void
 freeline (struct line *line)
 {
-  free ((char *) line->fields);
+  free (line->fields);
   free (line->buf.buffer);
   line->buf.buffer = NULL;
 }
@@ -287,7 +286,7 @@ initseq (struct seq *seq)
 {
   seq->count = 0;
   seq->alloc = 1;
-  seq->lines = (struct line *) xmalloc (seq->alloc * sizeof (struct line));
+  seq->lines = xmalloc (seq->alloc * sizeof (struct line));
 }
 
 /* Read a line from FP and add it to SEQ.  Return 0 if EOF, 1 otherwise.  */
@@ -299,7 +298,7 @@ getseq (FILE *fp, struct seq *seq)
     {
       seq->alloc *= 2;
       seq->lines = (struct line *)
-	xrealloc ((char *) seq->lines, seq->alloc * sizeof (struct line));
+	xrealloc (seq->lines, seq->alloc * sizeof (struct line));
     }
 
   if (get_line (fp, &seq->lines[seq->count]))
@@ -317,7 +316,7 @@ delseq (struct seq *seq)
   for (i = 0; i < seq->count; i++)
     if (seq->lines[i].buf.buffer)
       freeline (&seq->lines[i]);
-  free ((char *) seq->lines);
+  free (seq->lines);
 }
 
 /* Return <0 if the join field in LINE1 compares less than the one in LINE2;
@@ -608,7 +607,7 @@ add_field (int file, int field)
   assert (file == 0 || file == 1 || file == 2);
   assert (file == 0 ? field < 0 : field >= 0);
 
-  o = (struct outlist *) xmalloc (sizeof (struct outlist));
+  o = xmalloc (sizeof (struct outlist));
   o->file = file;
   o->field = field;
   o->next = NULL;
@@ -720,7 +719,7 @@ make_blank (struct line *blank, int count)
   blank->buf.buffer = xmalloc (blank->buf.size);
   buffer = (unsigned char *) blank->buf.buffer;
   blank->fields = fields =
-    (struct field *) xmalloc (sizeof (struct field) * count);
+    xmalloc (sizeof (struct field) * count);
   for (i = 0; i < count; i++)
     {
       buffer[i] = '\t';
@@ -737,6 +736,7 @@ main (int argc, char **argv)
   FILE *fp1, *fp2;
   int optc, prev_optc = 0, nfiles;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
