@@ -22,7 +22,6 @@
 #include <sys/types.h>
 
 #include "system.h"
-#include "closeout.h"
 #include "long-options.h"
 #include "error.h"
 #include "xgetcwd.h"
@@ -60,6 +59,7 @@ main (int argc, char **argv)
 {
   char *wd;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -70,7 +70,7 @@ main (int argc, char **argv)
   parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
 		      AUTHORS, usage);
 
-  if (argc != 1)
+  if (1 < argc)
     error (0, 0, _("ignoring non-option arguments"));
 
   wd = xgetcwd ();
