@@ -336,6 +336,12 @@
 #endif
 
 #include "gettext.h"
+#if ! ENABLE_NLS
+# undef textdomain
+# define textdomain(Domainname) /* empty */
+# undef bindtextdomain
+# define bindtextdomain(Domainname, Dirname) /* empty */
+#endif
 
 #define _(msgid) gettext (msgid)
 #define N_(msgid) msgid
@@ -389,11 +395,6 @@ off_t lseek ();
 unsigned long strtoul ();
 #endif
 
-/* This is needed on some AIX systems.  */
-#if !HAVE_DECL_STRTOULL && HAVE_UNSIGNED_LONG_LONG
-unsigned long long strtoull ();
-#endif
-
 #if !HAVE_DECL_GETLOGIN
 char *getlogin ();
 #endif
