@@ -1,5 +1,5 @@
 /* tac - concatenate and print files in reverse
-   Copyright (C) 1988-1991, 1995-2002 Free Software Foundation, Inc.
+   Copyright (C) 1988-1991, 1995-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -41,7 +41,6 @@ tac -r -s '.\|
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
-#include "closeout.h"
 
 #include <regex.h>
 
@@ -227,7 +226,7 @@ tac_seekable (int input_fd, const char *file)
      in the input file. */
 
   if (lseek (input_fd, file_pos, SEEK_SET) < 0)
-    error (0, errno, "%s: seek failed", file);
+    error (0, errno, _("%s: seek failed"), file);
 
   if (safe_read (input_fd, G_buffer, saved_record_size) != saved_record_size)
     {
@@ -608,6 +607,7 @@ main (int argc, char **argv)
   int optc, errors;
   int have_read_stdin = 0;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
